from base_st_classes import *

class OpenattrSuite(NFSSuite):
    """Test operation 19: OPENATTR

    Equivalence partitioning:

    Input Condition: current filehandle
        Valid equivalence classes:
            file(1)
            dir(2)
            block(3)
            char(4)
            link(5)
            socket(6)
            FIFO(7)
        Invalid equivalence classes:
            attribute directory(8)
            named attribute(9)
    Input Condition: createdir
        Valid equivalence classes:
            false(20)
            true(21)
        Invalid equivalence classes:
            -
    """

    def setUp(self):
        NFSSuite.setUp(self)
        self.fh = None
        self.stateid = None

    def tearDown(self):
        self.ncl.close_file(self.fh, self.stateid, error=None)
        try: self.clean_dir(self.tmp_dir)
        except SkipException, e:
            pass

    def _get_ndir(self):
        """Return filehandle to a named attribute directory"""
        try: self.create_obj()
        except SkipException, e:
            self.skip(e)

        ops = [self.ncl.putrootfh_op()] + self.ncl.lookup_path(self.tmp_dir)
        ops.append(self.ncl.lookup_op(self.obj_name))
        ops.append(self.ncl.getfh_op())
        ops.append(self.ncl.openattr_op(TRUE))
        ops.append(self.ncl.getfh_op())
        
        res = self.ncl.do_ops(ops)
        self.assert_status(res, [NFS4_OK, NFS4ERR_NOTSUPP])
        if res.status == NFS4ERR_NOTSUPP:
            self.skip("OPENATTR not supported")
        return (res.resarray[-1].arm.arm.object,
                res.resarray[-3].arm.arm.object)


    #
    # Testcases covering valid equivalence classes.
    #
    def _openattr(self, createdir):
        for (lookupops,path) in zip(self.ncl.lookup_all_objects(),self.all_objects):
            ops = [self.ncl.putrootfh_op()] + lookupops
            ops.append(self.ncl.getfh_op())
            ops.append(self.ncl.openattr_op(createdir))
            ops.append(self.ncl.getfh_op())
            res = self.ncl.do_ops(ops)

            if res.status == NFS4ERR_NOTSUPP:
                self.info_message("OPENATTR not supported on " + str(path))

            self.assert_status(res, [NFS4_OK, NFS4ERR_NOTSUPP])

    def testValidNoCreate(self):
        """OPENATTR on all non-attribute objects, createdir=FALSE

        Covered valid equivalence classes: 1, 2, 3, 4, 5, 6, 7, 20
        """
        # FRED - shouldn't this respond _NOENT if there are no nattrs
        self._openattr(FALSE)

    def testValidCreate(self):
        """OPENATTR on all non-attribute objects, createdir=TRUE

        Covered valid equivalence classes: 1, 2, 3, 4, 5, 6, 7, 21
        """
        # FRED - this is modifying the test tree, should create temp files
        self._openattr(TRUE)

    def testFileHandle(self):
        """OPENATTR filehandle manipulation should work"""
        nfh, fh = self._get_ndir()
        ops = [self.ncl.putfh_op(nfh)]
        ops.append(self.ncl.getfh_op())
        ops.append(self.ncl.savefh_op())
        ops.append(self.ncl.putrootfh_op())
        ops.append(self.ncl.restorefh_op())
        ops.append(self.ncl.getfh_op())
        res = self.ncl.do_ops(ops)
        self.assert_OK(res)
        if res.resarray[-1].arm.arm.object != res.resarray[-5].arm.arm.object:
            self.fail("Could not save and restore nattr dir filehandle")
        
    def testLookupp(self):
        """OPENATTR: lookupp should point back to original file"""
        nfh, fh = self._get_ndir()
        ops = [self.ncl.putfh_op(nfh)]
        ops.append(self.ncl.lookupp_op())
        ops.append(self.ncl.getfh_op())
        res = self.ncl.do_ops(ops)
        self.assert_OK(res)
        if res.resarray[-1].arm.arm.object != fh:
            self.fail("Lookupp does not return original fh")

    def testReaddir(self):
        """OPENATTR: readdir should list nattrs"""
        nfh, fh = self._get_ndir()
        ops = [self.ncl.putfh_op(nfh)]
        readdirop = self.ncl.readdir_op(cookie=0, cookieverf="\x00",
                                        dircount=0, maxcount=4096,
                                        attr_request=[])
        ops.append(readdirop)
        res = self.ncl.do_ops(ops)
        self.assert_OK(res)
        
        
    #
    # Testcases covering invalid equivalence classes.
    #
    def testOnAttrDir(self):
        """OPENATTR on attribute directory should fail with NFS4ERR_INVAL

        Covered invalid equivalence classes: 8
        """
        nfh, fh = self._get_ndir()
        ops = [self.ncl.putfh_op(nfh)]
        ops.append(self.ncl.openattr_op(TRUE))
        res = self.ncl.do_ops(ops)
        self.assert_status(res, [NFS4ERR_INVAL])

    def testExclusive(self):
        """OPENATTR: an exclusive open of an nattr should return NFS4ERR_INVAL
        """
        self.init_connection()
        nfh, fh = self._get_ndir()
        ops = [self.ncl.putfh_op(nfh)]
        ops.append(self.ncl.open('garbageattr',OPEN4_CREATE, EXCLUSIVE4,
                                 nfs4lib.dict2fattr({FATTR4_MODE: 0644 }),
                                 0x12345678, OPEN4_SHARE_ACCESS_BOTH,
                                 OPEN4_SHARE_DENY_NONE))
        ops.append(self.ncl.getfh_op())
        res = self.ncl.do_ops(ops)
        self.assert_status(res, [NFS4ERR_INVAL])
                   
    def testScratch(self):
        """OPENATTR test

        """
        # Open attribute dir for root dir
        ops = [self.ncl.putrootfh_op()] + [self.ncl.lookup_op('foo')]
        ops.append(self.ncl.getfh_op())
        ops.append(self.ncl.openattr_op(FALSE))
        readdirop = self.ncl.readdir_op(cookie=0, cookieverf="\x00",
                                        dircount=0, maxcount=4096,
                                        attr_request=[])
        ops.append(self.ncl.getfh_op())
        ops.append(readdirop)
        ops.append(self.ncl.lookup_op("user.test"))
        ops.append(self.ncl.getfh_op())
        ops.append(self.ncl.read(stateid4(0, ""),0,10))
        ops.append(self.ncl.getattr([FATTR4_SIZE]))
        res = self.ncl.do_ops(ops)
        self.assert_OK(res)

    def testOnAttr(self):
        """OPENATTR on attribute should fail with NFS4ERR_INVAL

        Covered invalid equivalence classes: 9

        Comments: Not yet implemented. 
        """
        # Open attribute dir for doc/README
        self.skip("Not implemented")
        lookupops = self.ncl.lookup_path(self.regfile)
        ops = [self.ncl.putrootfh_op()] + lookupops
        ops.append(self.ncl.openattr_op(FALSE))
        res = self.ncl.do_ops(ops)

        if res.status == NFS4ERR_NOTSUPP:
            self.info_message("OPENATTR not supported on %s, cannot try this test" \
                              % self.regfile)
            return

        # FIXME: Implement rest of testcase.
        self.info_message("(TEST NOT IMPLEMENTED)")

    def testNoFh(self):
        """OPENATTR without (cfh) should return NFS4ERR_NOFILEHANDLE """
        res = self.ncl.do_ops([self.ncl.openattr_op(FALSE)])
        self.assert_status(res, [NFS4ERR_NOFILEHANDLE])
        res = self.ncl.do_ops([self.ncl.openattr_op(TRUE)])
        self.assert_status(res, [NFS4ERR_NOFILEHANDLE])

